<?php
class ControllerPaymentIpasspay extends Controller {
	public function index() {
		$this->load->language('payment/ipasspay');

		$data['text_credit_card'] = $this->language->get('text_credit_card');
		$data['text_loading'] = $this->language->get('text_loading');
		$data['text_card_type'] = $this->language->get('text_card_type');
		$data['text_card_name'] = $this->language->get('text_card_name');
		$data['text_card_digits'] = $this->language->get('text_card_digits');
		$data['text_card_expiry'] = $this->language->get('text_card_expiry');
		$data['text_confirm_delete'] = $this->language->get('text_confirm_delete');

		$data['entry_card'] = $this->language->get('entry_card');
		$data['entry_card_existing'] = $this->language->get('entry_card_existing');
		$data['entry_card_new'] = $this->language->get('entry_card_new');
		$data['entry_card_save'] = $this->language->get('entry_card_save');
		$data['entry_cc_owner'] = $this->language->get('entry_cc_owner');
		$data['entry_cc_type'] = $this->language->get('entry_cc_type');
		$data['entry_cc_number'] = $this->language->get('entry_cc_number');
		$data['entry_cc_expire_date'] = $this->language->get('entry_cc_expire_date');
		$data['entry_cc_cvv2'] = $this->language->get('entry_cc_cvv2');
		$data['entry_cc_choice'] = $this->language->get('entry_cc_choice');

		$data['button_confirm'] = $this->language->get('button_confirm');
		$data['button_delete_card'] = $this->language->get('button_delete_card');

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];
			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		$data['months'] = array();

		for ($i = 1; $i <= 12; $i++) {
			$data['months'][] = array(
				'text' => strftime('%B', mktime(0, 0, 0, $i, 1, 2000)),
				'value' => sprintf('%02d', $i),
			);
		}

		$today = getdate();

		$data['year_valid'] = array();

		for ($i = $today['year'] - 10; $i < $today['year'] + 1; $i++) {
			$data['year_valid'][] = array(
				'text' => strftime('%Y', mktime(0, 0, 0, 1, 1, $i)),
				'value' => strftime('%Y', mktime(0, 0, 0, 1, 1, $i)),
			);
		}

		$data['year_expire'] = array();

		for ($i = $today['year']; $i < $today['year'] + 11; $i++) {
			$data['year_expire'][] = array(
				'text' => strftime('%Y', mktime(0, 0, 0, 1, 1, $i)),
				'value' => strftime('%Y', mktime(0, 0, 0, 1, 1, $i)),
			);
		}

		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/ipasspay.tpl')) {
			return $this->load->view($this->config->get('config_template') . '/template/payment/ipasspay.tpl', $data);
		} else {
			return $this->load->view('default/template/payment/ipasspay.tpl', $data);
		}
	}

/**
 * the function for creating uuid in PHP
 */
	function create_guid() {
		$charid = strtoupper(md5(uniqid(mt_rand(), true)));
		$hyphen = chr(45); // "-"
		$uuid = substr($charid, 0, 8) . $hyphen
		. substr($charid, 8, 4) . $hyphen
		. substr($charid, 12, 4) . $hyphen
		. substr($charid, 16, 4) . $hyphen
		. substr($charid, 20, 12);
		return $uuid;
	}

	public function send() {

		$reg_cvv = "/^\d{3}$/";

		$cc_validate = new cc_validation();
		$result = $cc_validate->validate($this->request->post['cc_number'],
			$this->request->post['cc_expire_date_month'],
			substr($this->request->post['cc_expire_date_year'], 2));

		switch ($result) {
		case -1:
			$json['error'] = "Invalid card number";
			break;
		case -2:
		case -3:
		case -4:
			$json['error'] = "Invalid expiry date";
			break;
		case false:
			$json['error'] = "Invalid card number";
			break;
		}

		if (($result == false) || ($result < 1)) {
			exit(json_encode($json));
		}

		if (!preg_match($reg_cvv, $this->request->post['cc_cvv2'])) {
			$json['error'] = "Invalid CVV number";
			exit(json_encode($json));
		}
		$json['error'] = 0;

		$this->load->language('payment/ipasspay');
		$this->load->model('checkout/order');
		$this->load->model('payment/ipasspay');
		$this->load->model('account/order');

		$gateway_url = $this->config->get("ipasspay_gateway_url");
		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		$params['merchantid'] = $this->config->get("ipasspay_mid");
		$params['siteid'] = $this->config->get("ipasspay_site_id");
		$params['invoiceid'] = $this->session->data['order_id'];
		$params['amount'] = number_format($this->currency->format($order_info['total'], $order_info['currency_code'], false, false), 2, '.', '');
		$params['currency'] = $this->session->data['currency'];
		$api_key = $this->config->get("ipasspay_api_key");
		$gateway_version = $this->config->get("ipasspay_gateway_version");

		//encrypted the sensitive data
		$hash_info = hash("sha256", $params['merchantid'] . $params['siteid'] .
			$params['invoiceid'] . $params['amount'] . $params['currency'] . $api_key);

		$curlPost = "order_amount=" . $params['amount'];
		$curlPost .= "&order_currency=" . $params['currency'];
		$curlPost .= "&mid=" . $params['merchantid']; //商户ID
		$curlPost .= "&site_id=" . $params['siteid']; //网站ID
		$curlPost .= "&oid=" . $params['invoiceid']; //订单ID
		$curlPost .= "&hash_info=" . $hash_info;

		$curlPost .= "&card_no=" . $this->request->post['cc_number']; //信用卡卡号
		$curlPost .= "&card_ex_year=" . substr($this->request->post['cc_expire_date_year'], 2); //信用卡过期年
		$curlPost .= "&card_ex_month=" . $this->request->post['cc_expire_date_month']; //信用卡过期月
		$curlPost .= "&card_cvv=" . $this->request->post['cc_cvv2']; //信用卡CVV

		$curlPost .= '&bill_firstname=' . substr($order_info['payment_firstname'], 0, 20);
		$curlPost .= '&bill_lastname=' . substr($order_info['payment_lastname'], 0, 20);
		$curlPost .= '&bill_street=' . substr($order_info['payment_address_1'], 0, 100);
		$curlPost .= '&bill_city=' . substr($order_info['payment_city'], 0, 40);
		$curlPost .= '&bill_state=' . ($order_info['payment_iso_code_2'] != 'US' ? $order_info['payment_zone'] : $order_info['payment_zone_code']);
		$curlPost .= '&bill_country=' . $order_info['payment_iso_code_2'];
		$curlPost .= '&bill_zip=' . substr($order_info['payment_postcode'], 0, 10);
		$curlPost .= '&bill_phone=' . substr($order_info['telephone'], 0, 20);
		$curlPost .= '&bill_email=' . substr($order_info['email'], 0, 255);

		$curlPost .= '&syn_url=' . $_SERVER["HTTP_REFERER"];
		$curlPost .= '&asyn_url=' . $_SERVER["HTTP_REFERER"];

		$curlPost .= '&source_ip=' . $this->request->server['REMOTE_ADDR']; //来源IP地址
		$curlPost .= '&source_url=' . $_SERVER["HTTP_REFERER"]; //来源URL地址
		$curlPost .= '&gateway_version=' . $gateway_version; //网关版本号
		$curlPost .= '&uuid=' . $this->create_guid(); //uuid

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $gateway_url);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $curlPost);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		$response = curl_exec($ch);
		curl_close($ch);

		//process $response
		$result = json_decode($response, true);

		switch ($result['status']) {
		case '1':
			# code...
			if ($result['data']['order_status'] == 2 || $result['data']['order_status'] == 3) {
				$message = "Payment success";
				$this->model_checkout_order->addOrderHistory($order_info['order_id'], $this->config->get('ipasspay_order_status_id'), $message);
				$json['error'] = "";
				$json['success'] = $this->url->link('checkout/success');
			} else {
				$json['error'] = $result['info'];
			}
			break;
		case '0':
			$json['error'] = $result['info'];
			break;
		default:
			$json['error'] = "Invalid response,please try again...";
		}
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}

	public function callback() {
	}

}

/**
 * cc_validation Class.
 * Class to validate credit card numbers etc
 *
 * @package classes
 */
class cc_validation {
	var $cc_type, $cc_number, $cc_expiry_month, $cc_expiry_year;

	function validate($number, $expiry_m, $expiry_y, $start_m = null, $start_y = null) {
		$this->cc_number = preg_replace('/[^0-9]/', '', $number);
		// NOTE: We check Solo before Maestro, and Maestro/Switch *before* we check Visa/Mastercard, so we don't have to rule-out numerous types from V/MC matching rules.
		if (preg_match('/^(6334[5-9][0-9]|6767[0-9]{2})[0-9]{10}([0-9]{2,3}?)?$/', $this->cc_number)) {
			$this->cc_type = "Solo"; // is also a Maestro product
		} else if (preg_match('/^(49369[8-9]|490303|6333[0-4][0-9]|6759[0-9]{2}|5[0678][0-9]{4}|6[0-9][02-9][02-9][0-9]{2})[0-9]{6,13}?$/', $this->cc_number)) {
			$this->cc_type = "Maestro";
		} else if (preg_match('/^(49030[2-9]|49033[5-9]|4905[0-9]{2}|49110[1-2]|49117[4-9]|49918[0-2]|4936[0-9]{2}|564182|6333[0-4][0-9])[0-9]{10}([0-9]{2,3}?)?$/', $this->cc_number)) {
			$this->cc_type = "Maestro"; // SWITCH is now Maestro
		} elseif (preg_match('/^4[0-9]{12}([0-9]{3})?$/', $this->cc_number)) {
			$this->cc_type = 'Visa';
		} elseif (preg_match('/^5[1-5][0-9]{14}$/', $this->cc_number)) {
			$this->cc_type = 'MasterCard';
		} elseif (preg_match('/^3[47][0-9]{13}$/', $this->cc_number)) {
			$this->cc_type = 'American Express';
		} elseif (preg_match('/^3(0[0-5]|[68][0-9])[0-9]{11}$/', $this->cc_number)) {
			$this->cc_type = 'Diners Club';
		} elseif (preg_match('/^(6011[0-9]{12}|622[1-9][0-9]{12}|64[4-9][0-9]{13}|65[0-9]{14})$/', $this->cc_number)) {
			$this->cc_type = 'Discover';
		} elseif (preg_match('/^(35(28|29|[3-8][0-9])[0-9]{12}|2131[0-9]{11}|1800[0-9]{11})$/', $this->cc_number)) {
			$this->cc_type = "JCB";
		} elseif (preg_match('/^5610[0-9]{12}$/', $this->cc_number)) {
			$this->cc_type = 'Australian BankCard'; // NOTE: is now obsolete
		} else {
			return -1;
		}

		if (is_numeric($expiry_m) && ($expiry_m > 0) && ($expiry_m < 13)) {
			$this->cc_expiry_month = $expiry_m;
		} else {
			return -2;
		}

		$current_year = date('Y');
		if (strlen($expiry_y) == 2) {
			$expiry_y = intval(substr($current_year, 0, 2) . $expiry_y);
		}

		if (is_numeric($expiry_y) && ($expiry_y >= $current_year) && ($expiry_y <= ($current_year + 10))) {
			$this->cc_expiry_year = $expiry_y;
		} else {
			return -3;
		}

		if ($expiry_y == $current_year) {
			if ($expiry_m < date('n')) {
				return -4;
			}
		}

		// check the issue month & year but only for Switch/Solo cards
		if (($start_m || $start_y) && in_array($this->cc_type, array('Switch', 'Solo'))) {
			if (!(is_numeric($start_m) && ($start_m > 0) && ($start_m < 13))) {
				return -2;
			}

			if (strlen($start_y) == 2) {
				if ($start_y > 80) {
					$start_y = intval('19' . $start_y);
				} else {
					$start_y = intval('20' . $start_y);
				}
			}

			if (!is_numeric($start_y) || ($start_y > $current_year)) {
				return -3;
			}
			if (!($start_y >= ($current_year - 10))) {
				return -3;
			}
		}
		return $this->is_valid();
	}

	function is_valid() {
		$cardNumber = strrev($this->cc_number);
		$numSum = 0;

		for ($i = 0; $i < strlen($cardNumber); $i++) {
			$currentNum = substr($cardNumber, $i, 1);

			// Double every second digit
			if ($i % 2 == 1) {
				$currentNum *= 2;
			}

			// Add digits of 2-digit numbers together
			if ($currentNum > 9) {
				$firstNum = $currentNum % 10;
				$secondNum = ($currentNum - $firstNum) / 10;
				$currentNum = $firstNum + $secondNum;
			}

			$numSum += $currentNum;
		}

		// If the total has no remainder it's OK
		return ($numSum % 10 == 0);
	}
}